function dat = AtableSolveEquations(cdat,tdat,varargin)
% function dat = AtableSolveEquations(cdat,tdat,varargin)
%    Solve for A-space line integrals using two stage kinear->Aslin->gridfit->lookup table. Then use table lookup
%    This version uses linear MLE for 1st stage
%    clips out of bounds to the calibration region. Uses 1.2X magnify for making table with gridfit
%    inputs:
%      cdat: a struct with calibration data with fields
%        cdat.logI: a matrix of log(I/I0) for each spectrum
%        As: the A values for each row of cdat.logI
%      tdat: a struct with object data with fields
%        logI: a matrix of log(I/I0) for each point for all measurement spectra
%        As: the A values for each row of logI
%      stats: (optional = []) struct with fields RL: Covariance of logI data. If not empty, used to compute initial estimate. 
%      solvedat: (optional = []) If specified uses this struct to solve the eqns instead of doing nlinfits 
%  NOTE: in the initial call specify stats. This is copied to solvedat so subsequent calls do not need stats. If
%  solvedat if present, its stats struct overrides a value passed as a parameter
%      ntable: (optional = 150) The number of points in the interpolation table
%    outputs:
%      dat: a struct with fields
%        cdat,tdat--the input data
%        Astest: the A values for the test data (tdat)
%        solvedat: a struct with parameters so can fit other sets of data without re-doing the calibration
%           See code for fields
% Example of use
%    % create the input data structs
% cdat.logI = logI;
% cdat.As = ts;
% tdat = cdat;
%    % solve for fit params and test with tdat
% dat = AtableSolveEquations(cdat,tdat);
%   % solve again without re-calibrating
% dat2 = AtableSolveEquations([],tdat,'solvedat',dat.solvedat);
% REA 6/27/09   
% REA 7/8/09 handle out of bounds points 
% REA 7/10/09 use 2nd order polynomial for initial fit
% REA 2/4/10 change algorithm to handle out of bounds points
nreqargs = 2;
assert(nargin>=nreqargs);
solvedat = [];
ntable = 150;
stats = [];

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'solvedat';         solvedat=varargin{i+1};  assert(isstruct(solvedat));   i=i+1;
     case 'ntable';         ntable=varargin{i+1};  assert(isscalar(ntable));   i=i+1;
     case 'stats';         stats=varargin{i+1}; assert(isstruct(stats));   i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

if isempty(solvedat)
      if isfield(cdat,'RL') 
         RL = cdat.RL;
      elseif ~isempty(stats)
         RL = stats.RL;
      else
         error('cannot find RL in cdat and not stats parameter');
      end
%      dat_centered = cdat.logI - repmat(stats.ave(:)',size(cdat.logI,1),1);
            % solve for M matrix from calibration data
      Mtranspose = cdat.As\cdat.logI;
      M = Mtranspose';
             % compute MLE inverse solution matrix from M and the covariance
     RLi = inv(RL);
     cslin = inv(M'*RLi*M)*M'*RLi;
     As_initfit = cdat.logI*(cslin');
    dAs =  cdat.As - As_initfit;
    magnify = 1.2;
    [~,~,dAs1] = MakeInterpolationTableLoc([As_initfit dAs(:,1)],ntable,ntable,'magnify',magnify);
    [xv,yv,dAs2,zcorners] = MakeInterpolationTableLoc([As_initfit dAs(:,2)],ntable,ntable,'magnify',magnify);
    [xs,ys] = meshgrid(xv,yv);
    solvedat.estimator = 'AtableSolveEquations';
    solvedat.stats = stats;
    solvedat.cdat = cdat;
    solvedat.cslin = cslin;
    solvedat.xs = xs;
    solvedat.ys = ys;
    solvedat.dAs = dAs;
    solvedat.dAs1 = dAs1;
    solvedat.dAs2 = dAs2;
    solvedat.zdAs = reshape(zcomp([dAs1(:) dAs2(:)]),size(dAs1));
    solvedat.zcorners = zcorners;
    solvedat.M = M;
    solvedat.RL = RL;
end
dat.solvedat = solvedat;

dat.tdat = tdat;
As_initfit_test = tdat.logI*(solvedat.cslin)';

   % solve for estimated dAs and add to initfit
xs = solvedat.xs;
ys = solvedat.ys;
[zpnts,idx_clipped] = ClipPoints2BoxLoc(complex(As_initfit_test(:,1), As_initfit_test(:,2)),solvedat.zcorners); 
dat.idx_clipped = idx_clipped;

zdAstest = interp2(xs,ys,solvedat.zdAs,real(zpnts(:)),imag(zpnts(:)));
   
dat.Astest = As_initfit_test + zcomp(zdAstest);
dat.Asdiff = tdat.As - dat.Astest;
dat.errs_raw =  std(tdat.As - dat.Astest);
dat.errs = std(tdat.As - dat.Astest)./range(tdat.As);
dat.errs_init = std(tdat.As - As_initfit_test)./range(tdat.As);

    %------------ Local Functions ---------------
    function [xv,yv,Zinterp,zrng] = MakeInterpolationTableLoc(xyz,nxinterp,nyinterp,varargin)
    %  function xyz_interp = MakeInterpolationTable(xyz,nxinterp,nyinterp)
    %  Make an interpolation table from a set of unevenly spaced points
    %  inputs:
    %    xyz: a size (npoints,3) matrix giving the x,y coordinates 
    %            in first 2 columns and zcoords in last column
    %    nxinterp,nyinterp: the number of points in each dimension of the interpolated data
    %    magnify: (optional = 1.0) The magnification of the range of input values. Uses
    %       the extrapolate capability of gridfit
    %  outputs:
    %    xv,yv: vectors of the x and y coordinates on the points of the 2D grid
    %    Zinterp = size(nyinterp,nxinterp) matrix of the interpolated points on an evenly spaced grid
    %    zrng: the 4 corners of the interpolate region
    %  REA 6/21/07-6/29/09
    
      % check arguments
    nreqargs = 3;
    assert(nargin>=nreqargs);
    magnify = 1.0;
    
    if(nargin>nreqargs)
      i=1;
      while(i<=size(varargin,2))
         switch lower(varargin{i})
    %     case 'max';          minmax=1;
         case 'magnify';  magnify=varargin{i+1};  assert(isscalar(magnify)&&isreal(magnify)&&(magnify>0));   i=i+1;
        otherwise
          error('Unknown argument %s given',varargin{i});
         end
         i=i+1;
      end
    end
    
    assert(size(xyz,2)==3);
    zs = xyz(:,1) +1i*xyz(:,2);
    zcoords = xyz(:,3);
    
        % Determine the minimum and the maximum x and y values:
    zcorners4 = zrange2(zs);
    zrng =  ContourMagnifyZ(zcorners4,magnify);
    
        % Define the range and spacing of the x- and y-coordinates, and then fit them into X and Y
    xv = linspace(real(zrng(1)),real(zrng(2)), nxinterp);
    yv = linspace(imag(zrng(1)),imag(zrng(3)), nyinterp);
    
        % Calculate Z in the X-Y interpolation space, which is an evenly spaced grid:
    Zinterp = gridfit(real(zs),imag(zs),zcoords,xv,yv,'smoothness',0.1); % let gridfit extrapolate
    
    function [zsclipped,idxclipped,idx_nearest_side] = ClipPoints2BoxLoc(zs,zcorners)
    % clip a 2D set of points to the nearest points on the sides of a box
%    Chooses the side that is closest to the point
%    inputs:
%      zs: the points 2 clip
%      zcorners: the 4 corners of the box. May be output of zrange2
%    outputs:
%      zsclipped: the clipped points. The original points if inside the box, else the points clipped to the box sides 
%      idxclipped: the indexes of the points that are clipped
%      idx_nearest_side: the index of the nearest side of the points clipped (1:4)
    zsclipped = zs; 
    idxclipped = find(~isinpolyz(zs,zcorners));
    idx_nearest_side = idxclipped; % do this just to pre-define an array of same length
    if ~isempty(idxclipped)
       zcenter = mean(zcorners);
       zbox = [zcorners(:); zcorners(1)]; % close the box
       for k = 1:numel(idxclipped)
         zseg = [zcenter zs(idxclipped(k))];
         xidx = IntersectSegPolyline(zseg,zbox);
         xidx = xidx(xidx>0);
         if isempty(xidx)
             error('AtableSolveEquations: ClipPoints2BoxLoc was not able to clip point');
         end
         zsclipped(idxclipped(k)) = interp1(1:5, zbox, xidx(1));
         idx_nearest_side(k) = floor(xidx);
       end
    end
       

        