function dat = NeuralNetEstimator(cdat,tdat,varargin)
% function dat = NeuralNetEstimator(cdat,tdat,varargin)
%    Estimate A-vectors using a neural network estimator
%    inputs:
%      cdat: a struct with calibration data with fields
%        cdat.logI: a matrix of log(I/I0) for each spectrum
%        As: the A vectors for each row of cdat.logI
%      tdat: a struct with object data with fields
%        logI: a matrix of log(I/I0) for each point for all measurement spectra
%        As: the A vectors for each row of logI
%      solvedat: (optional = []) If specified uses struct to make estimates 
%    outputs:
%      dat: a struct with fields
%        cdat,tdat--the input data
%        Astest: the A vectors for the test data (tdat)
%        solvedat: a struct with parameters so can make estimates without re-doing the network create
%           See code for fields
% Example of use
%    % create the input data structs
% cdat.logI = logI;
% cdat.As = ts;
% tdat = cdat;
%    % solve for fit params and test with tdat
% dat = NeuralNetEstimator(cdat,tdat);
%   % solve again without re-computing net
% dat2 = NeuralNetEstimator([],tdat,'solvedat',dat.solvedat);
% 14/Apr/2015 11:20 - 21/Apr/2015 8:59
nreqargs = 2;
assert(nargin>=nreqargs);
solvedat = [];
use_builtin_net = false;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'solvedat';         solvedat=varargin{i+1};  assert(isstruct(solvedat));   i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

if isempty(solvedat)
       % compute the neural net
    % Choose a Training Function
    trainFcn = 'trainlm';  % Levenberg-Marquardt

    % Create a Fitting Network
    hiddenLayerSize = 3; % optimal number per Zimmerman and Schmidt paper.
    net = fitnet(hiddenLayerSize,trainFcn);

    % Setup Division of Data for Training, Validation, Testing
    net.divideParam.trainRatio = 70/100;
    net.divideParam.valRatio = 15/100;
    net.divideParam.testRatio = 15/100;

    % Train the Network
    % uncomment following line to display training window
    net.trainParam.showWindow = false;
    net = train(net,cdat.logI',cdat.As');
    solvedat.net = net;
end
   % neural net performance
if ~isempty(cdat)   
    solvedat.As_fit_train = solvedat.net(cdat.logI')';
    solvedat.As_fit_err = cdat.As - solvedat.As_fit_train;
    solvedat.cdat = cdat;
end    
solvedat.estimator =   'NeuralNetEstimator';  
dat.solvedat = solvedat;
dat.tdat = tdat;

    % compute the estimates with the neural net
dat.Astest = solvedat.net(tdat.logI')';
if isfield(tdat,'As')
    dat.Asdiff = tdat.As - dat.Astest;
    dat.errs_raw =  std(tdat.As - dat.Astest);
    dat.errs = std(tdat.As - dat.Astest)./range(tdat.As);
end